# -*- coding: utf-8 -*-

# Copyright 2015 X Studios Inc
#
# This file is part of proprietary software and use of this file
# is strictly prohibited without written consent.
#
# @author  Tim Santor  <tsantor@xstudios.agency>

"""Downloads a manifest and its required media files."""

# -----------------------------------------------------------------------------
# Prevent compatibility regressions
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

# Standard
import logging
import os
import requests

# 3rd Party
# from progressbar import ProgressBar

# App
from manifest import utils
from manifest import config

# -----------------------------------------------------------------------------


def download_file(url, filepath):
    """Downloads a file and saves it to disk."""
    req = requests.get(url, stream=True)

    with open(filepath, 'wb') as ofile:
        for chunk in req.iter_content(chunk_size=1024):
            if chunk:  # filter out keep-alive new chunks
                ofile.write(chunk)
                ofile.flush()
                os.fsync(ofile.fileno())


def run():
    """Run script."""
    logger = logging.getLogger(__name__)

    # Get some settings from the config
    api_url = config.get('default', 'api_url')
    base_url = config.get('default', 'base_url')
    keys = config.get('default', 'keys')
    local_dir = config.get('default', 'local_dir')
    media_dir = config.get('default', 'media_dir')
    post_cfg = dict(config.items('post_download'))

    # Make local_dir if it does not exist
    utils.make_dir(local_dir)

    # Backup existing manifest
    utils.backup_manifest(local_dir)

    # Get manifest from API endpoint
    parsed_json = utils.get_manifest(api_url)
    # print(parsed_json)

    # Save manifest
    utils.save_manifest(parsed_json, local_dir, base_url, media_dir)

    # Find all the keys that match the key we're looking for
    logger.debug('Building file list based on target key values...')
    find_keys = utils.get_keys(keys)
    file_list = utils.find_all(parsed_json, find_keys)
    # total_files = len(file_list)

    # Download files
    logger.debug('Downloading manifest files...')

    # Create a progress bar
    # pbar = ProgressBar(term_width=80, maxval=total_files).start()

    num_files = 0
    for filepath in file_list:
        # print('-'*80)

        # Strip out the base URL if any
        filepath = filepath.replace(base_url, '').lstrip('/')
        # print('filepath: ' + filepath)

        # Determine remote / local file names
        remote_file = os.path.join(base_url, filepath)
        local_file = filepath.replace(media_dir, '').lstrip('/')
        local_file = os.path.join(local_dir, local_file)
        # print('remote_file: ' + remote_file)
        # print('local_file: ' + local_file)
        # continue

        # Ensure the local dir structure matches our remote dir structure
        create_dir = os.path.dirname(local_file)
        # print('create_dir: ' + create_dir)
        # continue
        utils.make_dir(create_dir)

        # Download file
        download_file(remote_file, local_file)

        # Update progress bar
        num_files += 1
        # pbar.update(num_files)

    # Ensure progress bar in finished state
    # pbar.finish()

    # logger.debug('Time Elapsed: {0}'.format(pbar.seconds_elapsed))
    logger.debug('Downloaded {0} files'.format(num_files))

    # Launch an app
    utils.post_download(post_cfg)

# -----------------------------------------------------------------------------

if __name__ == "__main__":
    run()
