# -*- coding: utf-8 -*-

# Copyright 2015 X Studios Inc
#
# This file is part of proprietary software and use of this file
# is strictly prohibited without written consent.
#
# @author  Tim Santor  <tsantor@xstudios.agency>

"""Downloads a manifest and its required media files."""

# -----------------------------------------------------------------------------
# Prevent compatibility regressions
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

# Standard
import logging
import os
import paramiko
import pysftp
import sys
import urllib

# 3rd Party
# from progressbar import ProgressBar

# App
from manifest import utils
from manifest import config

# -----------------------------------------------------------------------------


def run():
    """Run script."""
    logger = logging.getLogger(__name__)

    # Get some settings from the config
    api_token = None
    if config.has_option('default', 'api_token'):
        api_token = config.get('default', 'api_token')
    api_url = config.get('default', 'api_url')
    base_url = config.get('default', 'base_url')
    keys = config.get('default', 'keys')
    local_dir = config.get('default', 'local_dir')
    media_dir = config.get('default', 'media_dir')
    post_cfg = dict(config.items('post_download'))
    sftp_cfg = dict(config.items('sftp'))

    # Make local_dir if it does not exist
    utils.make_dir(local_dir)

    # Backup existing manifest
    utils.backup_manifest(local_dir)

    # Get manifest from API endpoint
    parsed_json = utils.get_manifest(api_url, api_token)
    # print(parsed_json)

    # Save manifest
    utils.save_manifest(parsed_json, local_dir, base_url, media_dir)

    # Find all the keys that match the key we're looking for
    logger.debug('Building file list based on target key values...')
    find_keys = utils.get_keys(keys)
    file_list = utils.find_all(parsed_json, find_keys)
    # total_files = len(file_list)

    # Download files
    logger.debug('Downloading manifest files...')

    # Login via SFTP
    try:
        sftp = pysftp.Connection(sftp_cfg['server'],
                                 username=sftp_cfg['username'],
                                 password=sftp_cfg['password'])
    except paramiko.ssh_exception.AuthenticationException:
        logger.exception('SSH auth error')
        sys.exit(1)
    except paramiko.ssh_exception.SSHException:
        logger.exception('SSH error')
        sys.exit(1)

    # Create a progress bar
    # pbar = ProgressBar(term_width=80, maxval=total_files).start()

    # Download files
    num_files = 0
    for filepath in file_list:
        # Strip out the base URL if any
        filepath = filepath.replace(base_url, '').lstrip('/')
        filepath = urllib.unquote(filepath)
        # print('filepath: ' + filepath)

        # Determine remote / local file names
        remote_file = os.path.join(sftp_cfg['remote_dir'], filepath)
        local_file = filepath.replace(media_dir, '').lstrip('/')
        local_file = os.path.join(local_dir, local_file)
        # print('remote_file: ' + remote_file)
        # print('local_file: ' + local_file)
        # continue

        # Ensure the local dir structure matches our remote dir structure
        create_dir = os.path.dirname(local_file)
        # print('create_dir: ' + create_dir)
        # continue
        utils.make_dir(create_dir)

        # If the file we are about to download exists locally
        if os.path.isfile(local_file):
            try:
                remote_file_mtime = sftp.stat(remote_file).st_mtime
            except IOError as err:
                logger.error(str(err)+' (stat): '+remote_file)
                continue

            local_file_mtime = os.stat(local_file).st_mtime

            # Compare remote file and local file mtimes
            if remote_file_mtime != local_file_mtime:
                # Download the file
                try:
                    sftp.get(remote_file, local_file, preserve_mtime=True)
                    num_files += 1
                except IOError as err:
                    logger.error(str(err)+' (download): '+remote_file)
                    continue

        # We don't have the file locally
        else:
            # Download the file
            try:
                sftp.get(remote_file, local_file, preserve_mtime=True)
                num_files += 1
            except IOError as err:
                logger.error(str(err)+': '+remote_file)
                continue

        # Update progress bar
        # pbar.update(num_files)

    # Ensure progress bar in finished state
    # pbar.finish()

    # Close SFTP connection
    sftp.close()

    # logger.debug('Time Elapsed: {0}'.format(pbar.seconds_elapsed))
    logger.debug('Downloaded {0} files'.format(num_files))

    # Launch an app
    utils.post_download(post_cfg)

'''
def safe_run():
    """Run our script, but catch any exceptions so we can revert back
    to the state before the script was run."""
    try:
        run()
    except Exception:
        # TODO: runs even if no error
        print('revert to orig manifest')
        args = get_parser().parse_args()
        utils.revert_manifest(args)
        # re-raise last exception
        raise
'''
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    run()
