# -*- coding: utf-8 -*-

# Copyright 2015 X Studios Inc
#
# This file is part of proprietary software and use of this file
# is strictly prohibited without written consent.
#
# @author  Tim Santor  <tsantor@xstudios.agency>

"""Util methods for Manifest Downloader"""

# -----------------------------------------------------------------------------

# Prevent compatibility regressions
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

# Standard
import io
import json
import logging
import os
import platform
import shutil
import subprocess
import sys
import shlex
from subprocess import Popen, PIPE

# 3rd Party
import requests
from requests.exceptions import ConnectionError

# -----------------------------------------------------------------------------

logger = logging.getLogger(__name__)


def make_dir(dir):
    """Lazy shortcut to make a dir if it does not exist."""
    try:
        if not os.path.isdir(dir):
            os.makedirs(dir)
    except OSError as err:
        logger.error(str(err))
        sys.exit(str(err))


def launch_app(app_path):
    """Launch desired app."""
    logger.debug('Open app: "{}"'.format(os.path.normpath(app_path)))
    # print(sys.platform)
    try:
        if platform.system() == 'Windows':
            print('windows')
            # print(os.path.normpath(app_path))
            app_path = os.path.normpath(app_path)
            # app_path = '\'"'+app_path.replace('\\', '\\\\')+'"\''
            # print(app_path)
            # subprocess.Popen(r'"C:\\Program Files (x86)\\Internet Explorer\\iexplorer.exe"')
            os.system('start "{}"'.format(app_path))
        else:
            subprocess.Popen([app_path])

    except OSError as err:
        logger.error(str(err))
        sys.exit()


def run_cmd(cmd):
    """Run a command and return the status, standard output and error."""
    proc = Popen(shlex.split(cmd), stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    status = not bool(proc.returncode)
    if status:
        logger.error(stderr.decode('utf-8').strip())
    return (status, stdout, stderr)


def post_download(config):
    """Post download action. Run a command, launch an app or both."""
    if 'command' in config:
        run_command(config['command'])
        logger.debug(config['command'])

    if 'app_path' in config:
        launch_app(config['app_path'])
        logger.debug(config['app_path'])

    sys.exit()


def get_keys(keys):
    """Return clean keys."""
    # Split values by comma
    keys = keys.split(',')
    # Remove extra white space
    keys = [x.strip() for x in keys]
    return keys


def find_all(obj, keys):
    """Find all keys in parsed json. (http://bit.ly/1RnjFaa)"""
    results = []

    keys_found = []

    def _find_all(obj, key):
        """Internal method we call recursively."""
        if isinstance(obj, dict):
            for item in obj:
                if isinstance(obj[item], (list, dict)):
                    _find_all(obj[item], key)
                elif item == key:
                    # print(item, obj[item])
                    results.append(obj[item])
                    # Keep track of keys we found for logging purposes
                    if item not in keys_found:
                        keys_found.append(item)
        elif isinstance(obj, list):
            for item in obj:
                if isinstance(item, (list, dict)):
                    _find_all(item, key)

    # Recursively search for all keys
    for k in keys:
        _find_all(obj, k)

    # Remove None values we may get for keys that are not found
    results = [x for x in results if x is not None]

    # Filter out only unique paths (especially handy with HTTP ingest)
    results = set(results)

    keys_found = ", ".join(keys_found)
    logger.debug('Found {0} unique files matching keys: {1}'.format(len(results), keys_found))

    # Debug
    # for path in results:
    #    print(path)

    return results


def get_manifest(url, token=None):
    """Returns manifest as JSON."""
    logger.debug('Getting manifest...')
    payload = {'format': 'json'}
    headers = {}

    if token:
        headers = {'authorization': 'Token {}'.format(token)}

    try:
        response = requests.get(url, params=payload, headers=headers)
        if response.status_code != 200:
            raise Exception(response.text)

    except (ConnectionError, Exception) as err:
        logger.error(str(err))
        # post_download(args)
        sys.exit(1)

    logger.debug(response.request.url)

    try:
        return response.json()
    except ValueError as err:
        logger.error(str(err))
        # post_download(args)
        sys.exit(1)


def backup_manifest(local_dir):
    """Backup current manifest."""
    filepath = os.path.join(local_dir, 'manifest.json')
    filepath_bak = filepath+'.bak'
    if os.path.exists(filepath):
        logger.debug('Backing up current manifest...')
        shutil.copyfile(filepath, filepath_bak)
        logger.debug(filepath_bak)


def save_manifest(json_content, local_dir, base_url, media_dir):
    """Saves manifest."""
    logger.debug('Saving manifest...')
    filepath = os.path.join(local_dir, 'manifest.json')

    # Save it human readable
    with io.open(filepath, 'w', encoding='utf-8') as myfile:
        data = json.dumps(json_content,
                          sort_keys=True,
                          indent=4,
                          separators=(',', ': '))

        # Strip the media URL from the file paths
        media_url_prefix = os.path.join(base_url, media_dir, '')
        data = data.replace(media_url_prefix, '')

        media_url_prefix = os.path.join(media_dir, '')
        data = data.replace('/' + media_url_prefix, '')

        myfile.write(unicode(data))
        logger.debug(filepath)


def revert_manifest(args):
    """Revert to original manifest."""
    filepath_bak = os.path.join(args.local_dir, 'manifest.json.bak')
    filepath = os.path.join(args.local_dir, 'manifest.json')
    if os.path.exists(filepath_bak):
        logger.debug('Reverting to original manifest...')
        shutil.copyfile(filepath_bak, filepath)
        logger.debug(filepath)
